"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Game_Picture = void 0;
var Utils_1 = require("../core/Utils");
var Game_Picture = /** @class */ (function () {
    function Game_Picture() {
        this.initialize();
    }
    Game_Picture.prototype.initialize = function () {
        this.initBasic();
        this.initTarget();
        this.initTone();
        this.initRotation();
    };
    Game_Picture.prototype.name = function () {
        return this._name;
    };
    Game_Picture.prototype.origin = function () {
        return this._origin;
    };
    Game_Picture.prototype.x = function () {
        return this._x;
    };
    Game_Picture.prototype.y = function () {
        return this._y;
    };
    Game_Picture.prototype.scaleX = function () {
        return this._scaleX;
    };
    Game_Picture.prototype.scaleY = function () {
        return this._scaleY;
    };
    Game_Picture.prototype.opacity = function () {
        return this._opacity;
    };
    Game_Picture.prototype.blendMode = function () {
        return this._blendMode;
    };
    Game_Picture.prototype.tone = function () {
        return this._tone;
    };
    Game_Picture.prototype.angle = function () {
        return this._angle;
    };
    Game_Picture.prototype.initBasic = function () {
        this._name = "";
        this._origin = 0;
        this._x = 0;
        this._y = 0;
        this._scaleX = 100;
        this._scaleY = 100;
        this._opacity = 255;
        this._blendMode = 0;
    };
    Game_Picture.prototype.initTarget = function () {
        this._targetX = this._x;
        this._targetY = this._y;
        this._targetScaleX = this._scaleX;
        this._targetScaleY = this._scaleY;
        this._targetOpacity = this._opacity;
        this._duration = 0;
    };
    Game_Picture.prototype.initTone = function () {
        this._tone = null;
        this._toneTarget = null;
        this._toneDuration = 0;
    };
    Game_Picture.prototype.initRotation = function () {
        this._angle = 0;
        this._rotationSpeed = 0;
    };
    Game_Picture.prototype.show = function (name, origin, x, y, scaleX, scaleY, opacity, blendMode) {
        this._name = name;
        this._origin = origin;
        this._x = x;
        this._y = y;
        this._scaleX = scaleX;
        this._scaleY = scaleY;
        this._opacity = opacity;
        this._blendMode = blendMode;
        this.initTarget();
        this.initTone();
        this.initRotation();
    };
    Game_Picture.prototype.move = function (origin, x, y, scaleX, scaleY, opacity, blendMode, duration) {
        this._origin = origin;
        this._targetX = x;
        this._targetY = y;
        this._targetScaleX = scaleX;
        this._targetScaleY = scaleY;
        this._targetOpacity = opacity;
        this._blendMode = blendMode;
        this._duration = duration;
    };
    Game_Picture.prototype.rotate = function (speed) {
        this._rotationSpeed = speed;
    };
    Game_Picture.prototype.tint = function (tone, duration) {
        if (!this._tone) {
            this._tone = [0, 0, 0, 0];
        }
        this._toneTarget = Utils_1.Utils.cloneArray(tone);
        this._toneDuration = duration;
        if (this._toneDuration === 0) {
            this._tone = Utils_1.Utils.cloneArray(this._toneTarget);
        }
    };
    Game_Picture.prototype.erase = function () {
        this._name = "";
        this._origin = 0;
        this.initTarget();
        this.initTone();
        this.initRotation();
    };
    Game_Picture.prototype.update = function () {
        this.updateMove();
        this.updateTone();
        this.updateRotation();
    };
    Game_Picture.prototype.updateMove = function () {
        if (this._duration > 0) {
            var d = this._duration;
            this._x = (this._x * (d - 1) + this._targetX) / d;
            this._y = (this._y * (d - 1) + this._targetY) / d;
            this._scaleX = (this._scaleX * (d - 1) + this._targetScaleX) / d;
            this._scaleY = (this._scaleY * (d - 1) + this._targetScaleY) / d;
            this._opacity = (this._opacity * (d - 1) + this._targetOpacity) / d;
            this._duration--;
        }
    };
    Game_Picture.prototype.updateTone = function () {
        if (this._toneDuration > 0) {
            var d = this._toneDuration;
            for (var i = 0; i < 4; i++) {
                this._tone[i] = (this._tone[i] * (d - 1) + this._toneTarget[i]) / d;
            }
            this._toneDuration--;
        }
    };
    Game_Picture.prototype.updateRotation = function () {
        if (this._rotationSpeed !== 0) {
            this._angle += this._rotationSpeed / 2;
        }
    };
    return Game_Picture;
}());
exports.Game_Picture = Game_Picture;
